﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote65_SmartThread_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private const int SAMPLECOUNT = 1000000;

        // Thread 이벤트가 발생했을 때 Count를 저장할 변수
        private int m_iCount;
        // 읽어온 ADC값을 저장할 변수
        private int[] m_ADCRawData;
        // PutData를 몇번 호출했을 때 Chart에 데이터를 출력할지 설정하는 변수
        private int m_iADCDrawChartStep;
        // 이전 Percentage 값을 저장할 변수
        private int m_iOldProgressPercentage;
        // List 템플릿 - 차트에 그릴 데이터 저장
        private List<int> m_listChartDatas = new List<int>();

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            labSampleCnt.Text = "Sampling Count : " + SAMPLECOUNT.ToString();
            radBackWorker.Checked = true;

            // UI Thread를 우회하여 호출하기 활성화
            smartThread1.WorkerReportsProgress = true;

            // Chart 설정
            smartDraw1.Chart.SetChartConfig();
            smartDraw1.BackLayer.ImageDraw(Properties.Resources.SmartDraw, 0, 0);

            SmartX.SmartMessageBox.Size(270, 160);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            smartThread1.End();
        }

        // Start 버튼을 클릭했을 때 발생하는 이벤트
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (smartThread1.State == SmartX.SmartThread.ThreadState.Stopped)
            {
                // SamplingCount 값 만큼 배열의 크기를 설정
                m_ADCRawData = new int[SAMPLECOUNT];
                // SamplingCount 값에서 1000을 나눠 DrawChartStep 값으로 설정
                m_iADCDrawChartStep = (SAMPLECOUNT / 1000);
                m_iCount = 0;
                m_iOldProgressPercentage = 0;
                smartProgressBar1.Value = 0;
                smartDraw1.Chart.PutDataAllClear();
                m_listChartDatas.Clear();
                smartThread1.Start();
            }
        }

        // Stop 버튼을 클릭했을 때 발생하는 이벤트
        private void btnStop_Click(object sender, EventArgs e)
        {
            if (smartThread1.State != SmartX.SmartThread.ThreadState.Stopped)
            {
                smartThread1.End();
            }
        }

        // Init 버튼을 클릭했을 때 발생하는 이벤트
        private void btnInit_Click(object sender, EventArgs e)
        {
            if (smartThread1.State != SmartX.SmartThread.ThreadState.Stopped)
            {
                smartThread1.End();
            }

            m_iCount = 0;
            m_iOldProgressPercentage = 0;
            smartProgressBar1.Value = 0;

            smartDraw1.Chart.PutDataAllClear();
            m_listChartDatas.Clear();
        }

        // 선택한 RadioButton에 따라 WorkerReportsProgress 속성을 설정하는 이벤트
        private void RadioButton_Click(object sender, EventArgs e)
        {
            if (sender == radBackWorker)
            {
                smartThread1.WorkerReportsProgress = true;
            }
            else
            {
                smartThread1.WorkerReportsProgress = false;
            }
        }

        // 빠른 연산처리가 가능한 Work Thread 이벤트에서 ADC값을 읽어오고 Data를 저장하는 처리를 진행합니다.
        private void smartThread1_ThreadFunction(SmartX.ThreadArgs args)
        {
            if (radBackWorker.Checked == true)
            {
                // Background Worker Thread로 반복적이면서 빠른 연산처리 - 시작 
                // UI thread 보다 빠르지만 사용자 인터페이스 직접 접근 불가능합니다.
                // 따라서 smartThread1.ReportProgress() 메서드를 호출하여 간접적으로 
                // 사용자 인터페이스를 갱신처리 합니다.

                // BackgoundProcessing 함수를 통해 전체에서 몇% 진행되었는지 받아와 저장
                int iProgressPercentage = BackgroundProcessing();

                // Return받은 Percentage값이 5% 단위로 증가되었을 때 UI를 갱신
                if (((iProgressPercentage % 5) == 0) && (m_iOldProgressPercentage != iProgressPercentage))
                {
                    // Background Worker Thread로 사용자 인터페이스 갱신 - UI Thread 간접 호출
                    smartThread1.ReportProgress(iProgressPercentage);
                    // 현재값을 이전값으로 저장하여 중복처리 방지
                    m_iOldProgressPercentage = iProgressPercentage;
                }
            }
        }

        // UI를 갱신할 수 있는 UI Thread 이벤트에서 SmartProgressBar, SmartDraw의 Chart를 갱신
        private void smartThread1_UIThreadFunction(SmartX.ThreadArgs args)
        {
            if (radUI.Checked == true)
            {
                // UI(UserInterface) Thread로 Background Worker Thread보다 느림 - 반복적인 연산처리 시작 
                // background worker thread 보다 UI thread가 느리지만 사용자 인터페이스를 적접 접근이 가능하다.
                int iProgressPercentage = BackgroundProcessing();

                if ((((iProgressPercentage % 5) == 0)) && (m_iOldProgressPercentage != iProgressPercentage))
                {
                    // 사용자 인터페이스 갱신
                    UIUpdate(iProgressPercentage);
                    // 현재값을 이전값으로 저장하여 중복처리 방지
                    m_iOldProgressPercentage = iProgressPercentage;
                }
            }
            else
            {
                // Work Thread에서 ReportProgress 메소드의 인자로 설정한 값을 받아와 UI 갱신 처리
                UIUpdate(args.iProgressPercentage);
            }
        }

        // 입력된 인자로 UI를 갱신하는 함수
        private void UIUpdate(int iPercent)
        {
            // SmartProgressBar의 Value값을 설정
            smartProgressBar1.Value = iPercent;

            // List에 추가된 Data들을 모두 Chart의 Data로 입력
            foreach (int iADData in m_listChartDatas)
            {
                smartDraw1.Chart.PutData(iADData);
            }
            // 현재까지 PutData 메소드를 통해 입력된 데이터를 모두 Chart로 출력
            smartDraw1.Chart.ChartNowDraw();
            // 다음 Data를 저장하기 위한 List 초기화
            m_listChartDatas.Clear();
        }

        // ADC값을 읽어와 배열에 저장하고 Chart를 그리기 위한 List 템플릿에 Item을 추가하는 함수
        private int BackgroundProcessing()
        {
            int iProgressPercentage;

            // ADC값을 읽어 배열에 저장 - Sampling Count만큼 ADC값이 저장됩니다.
            m_ADCRawData[m_iCount++] = smartADC1.ReadData(0);

            // Thread 발생횟수와 m_ADCDrawChartStep값을 나눠 나머지가 0이라면 ADC값을 연산처리 후 값을 List에 저장
            // Ex : SamplingCount = 10,000 일 때 m_iADCDrawChartStep의 값은 10이므로 10번 ADC값을 읽었을 때 Data를 저장
            if ((m_iCount % m_iADCDrawChartStep) == 0)
            {
                int iCalData;

                iCalData = (int)((211.0 / 4095.0) * (float)m_ADCRawData[m_iCount - 1]);
                m_listChartDatas.Add(iCalData);
            }

            // 전체 SmaplingCount 값중 몇%가 진행되었는지 변수에 저장 후 Return
            iProgressPercentage = (int)(((decimal)m_iCount / (m_ADCRawData.Length)) * 100.0m);

            return iProgressPercentage;
        }
    }
}